# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sim_msgs.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)


import base_msgs_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='sim_msgs.proto',
  package='AssisiMsg',
  serialized_pb='\n\x0esim_msgs.proto\x12\tAssisiMsg\x1a\x0f\x62\x61se_msgs.proto\"\xf6\x01\n\x05Spawn\x12!\n\x06header\x18\x01 \x01(\x0b\x32\x11.AssisiMsg.Header\x12\x0c\n\x04type\x18\x02 \x02(\t\x12\x0c\n\x04name\x18\x03 \x02(\t\x12\x1d\n\x04pose\x18\x04 \x02(\x0b\x32\x0f.AssisiMsg.Pose\x12\"\n\x08velocity\x18\x05 \x01(\x0b\x32\x10.AssisiMsg.Twist\x12#\n\x07polygon\x18\x06 \x01(\x0b\x32\x12.AssisiMsg.Polygon\x12%\n\x08\x63ylinder\x18\x07 \x01(\x0b\x32\x13.AssisiMsg.Cylinder\x12\x1f\n\x05\x63olor\x18\x08 \x01(\x0b\x32\x10.AssisiMsg.Color')




_SPAWN = _descriptor.Descriptor(
  name='Spawn',
  full_name='AssisiMsg.Spawn',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='AssisiMsg.Spawn.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='AssisiMsg.Spawn.type', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='name', full_name='AssisiMsg.Spawn.name', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pose', full_name='AssisiMsg.Spawn.pose', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='velocity', full_name='AssisiMsg.Spawn.velocity', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='polygon', full_name='AssisiMsg.Spawn.polygon', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='cylinder', full_name='AssisiMsg.Spawn.cylinder', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='color', full_name='AssisiMsg.Spawn.color', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=47,
  serialized_end=293,
)

_SPAWN.fields_by_name['header'].message_type = base_msgs_pb2._HEADER
_SPAWN.fields_by_name['pose'].message_type = base_msgs_pb2._POSE
_SPAWN.fields_by_name['velocity'].message_type = base_msgs_pb2._TWIST
_SPAWN.fields_by_name['polygon'].message_type = base_msgs_pb2._POLYGON
_SPAWN.fields_by_name['cylinder'].message_type = base_msgs_pb2._CYLINDER
_SPAWN.fields_by_name['color'].message_type = base_msgs_pb2._COLOR
DESCRIPTOR.message_types_by_name['Spawn'] = _SPAWN

class Spawn(_message.Message):
  __metaclass__ = _reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SPAWN

  # @@protoc_insertion_point(class_scope:AssisiMsg.Spawn)


# @@protoc_insertion_point(module_scope)
